@@ -48,9 +48,9 @@ class PCRE2Matcher : public PatternMatcher<std::string> {
  public:
   /// Convert a regex to an acceptable form, given the specified regex library signature `"[decls:]escapes[?+]"`, see reflex::convert.
   template<typename T>
-  static std::string convert(T regex, convert_flag_type flags = convert_flag::none)
+  static std::string convert(T regex, convert_flag_type flags = convert_flag::none, bool *multiline = NULL)
   {
-    return reflex::convert(regex, "imPRsx!#<>=&|'(0123456789*:abcdefghnrstvwxzABCDGHKNQRSVWXZ0123456789?+", flags);
+    return reflex::convert(regex, "imPRsx!#<>=&|'(0123456789*:abcdefghnrstvwxzABCDGHKNQRSVWXZ0123456789?+", flags, multiline);
   }
   /// Default constructor.
   PCRE2Matcher()
@@ -368,7 +368,12 @@ class PCRE2Matcher : public PatternMatcher<std::string> {
     /// @returns true when PCRE2 match found
   {
     if (pos_ == end_ && !eof_)
-      (void)peek_more();
+    {
+      if (method == Const::FIND)
+        set_current_and_peek_more(end_);
+      else
+        (void)peek_more();
+    }
     uint32_t flg = flg_;
     if (!eof_)
       flg |= PCRE2_PARTIAL_HARD;
@@ -458,9 +463,9 @@ class PCRE2UTFMatcher : public PCRE2Matcher {
  public:
   /// Convert a regex to an acceptable form, given the specified regex library signature `"[decls:]escapes[?+]"`, see reflex::convert.
   template<typename T>
-  static std::string convert(T regex, convert_flag_type flags = convert_flag::none)
+  static std::string convert(T regex, convert_flag_type flags = convert_flag::none, bool *multiline = NULL)
   {
-    return reflex::convert(regex, "imPRsx!#<>=&|'(0123456789*:abcdefghknprstvwxzABCDGHKNPQRSVWXZ0123456789?+", flags);
+    return reflex::convert(regex, "imPRsx!#<>=&|'(0123456789*:abcdefghknprstvwxzABCDGHKNPQRSVWXZ0123456789?+", flags, multiline);
   }
   /// Default constructor.
   PCRE2UTFMatcher() : PCRE2Matcher()
