@@ -573,10 +573,10 @@ class Output {
     mode_ |= FLUSH;
   }
 
-  // flush if output is line buffered to flush each line
-  void check_flush()
+  // flush if output is line buffered to flush each line, unless we have to wait to acquire the output lock
+  inline void check_flush()
   {
-    if (mode_ == FLUSH)
+    if (mode_ == FLUSH && (sync == NULL || sync->try_acquire(lock_)))
       flush();
   }
 
